{% extends "core_main_app/_render/admin/theme/tools/modal.html" %}

{% block modal_id %}operator-config-modal{% endblock %}
{% block modal_title %}<span id="operator-modal-action"></span> Search Operator{% endblock %}

{% block modal_body %}
{% include "core_explore_keyword_app/admin/search_ops_manager/modal/config/form.html" with form=data.operator_form %}
<span class="text-danger" id="operator-modal-form-error">&nbsp;</span>
{% endblock %}

{% block modal_footer %}
<button class="btn btn-secondary pull-left" {% if BOOTSTRAP_VERSION|first == "4" %}data-dismiss{% elif BOOTSTRAP_VERSION|first == "5"  %}data-bs-dismiss{% endif %}="modal">
    <i class="fas fa-times"></i> Cancel
</button>
<button id="btn-save-operator" class="btn btn-primary">
    <i class="fas fa-save"></i> Save
</button>
{% endblock %}