<div>
{% if data.error %}
    <div class="alert alert-danger"><i class="fas fa-times-circle"></i>
        {{ data.error }}
    </div>
{% else %}
    {% if data.warning %}
        <div class="alert alert-warning"><i class="fas fa-exclamation-triangle"></i>
            {{ data.warning }}
        </div>
    {% endif %}
    <form id="form_search" action="{% url 'core_explore_keyword_app_search' %}" method="POST">
        {% csrf_token %}
        {% include data.query_builder_interface %}
        <div class="d-inline-flex col-sm-12 extra-padding">
            <div class="col-sm-3 extra-padding">
                {% include data.data_sources_selector_template %}
                {% include 'core_explore_keyword_app/user/template_filter.html' %}
            </div>
            <div class="col-sm-9 extra-padding">
                {% include "core_explore_common_app/user/results/results.html" %}
            </div>
        </div>
    </form>
{% endif %}
{% include "core_explore_keyword_app/user/extras.html" %}
</div>
